#include <stdbool.h>

char *k8_bank_name(unsigned num);
void decode_amd_mc(enum cputype, struct mce *mce, int *ismemerr);
int mce_filter_amd(struct mce *m);
enum cputype select_amd_cputype(u32 family);

enum amdcpu {
	AMD_K8 = 0,
	AMD_F10H,
	AMD_F11H,
	AMD_F12H,
	AMD_F14H,
	AMD_F15H,
	AMD_F16H,
};

struct amd_decoder_ops {
	enum amdcpu cpu;
	bool (*mc0_mce)(u16, u8);
	bool (*mc1_mce)(u16, u8);
	bool (*mc2_mce)(u16, u8);
};

#define K8_MCE_THRESHOLD_BASE        (MCE_EXTENDED_BANK + 1)      /* MCE_AMD */
#define K8_MCE_THRESHOLD_TOP         (K8_MCE_THRESHOLD_BASE + 6 * 9)

#define K8_MCELOG_THRESHOLD_DRAM_ECC (4 * 9 + 0)
#define K8_MCELOG_THRESHOLD_LINK     (4 * 9 + 1)
#define K8_MCELOG_THRESHOLD_L3_CACHE (4 * 9 + 2)
#define K8_MCELOG_THRESHOLD_FBDIMM   (4 * 9 + 3)

#define BIT_64(n)                       (1ULL << (n))

#define EC(x)				((x) & 0xffff)
#define XEC(x, mask)			(((x) >> 16) & mask)

#define LOW_SYNDROME(x)			(((x) >> 15) & 0xff)
#define HIGH_SYNDROME(x)		(((x) >> 24) & 0xff)

#define TLB_ERROR(x)			(((x) & 0xFFF0) == 0x0010)
#define MEM_ERROR(x)			(((x) & 0xFF00) == 0x0100)
#define BUS_ERROR(x)			(((x) & 0xF800) == 0x0800)
#define INT_ERROR(x)			(((x) & 0xF4FF) == 0x0400)

#define TT(x)				(((x) >> 2) & 0x3)
#define TT_MSG(x)			transaction[TT(x)]
#define II(x)				(((x) >> 2) & 0x3)
#define II_MSG(x)			memoryio[II(x)]
#define LL(x)				((x) & 0x3)
#define LL_MSG(x)			cachelevel[LL(x)]
#define TO(x)				(((x) >> 8) & 0x1)
#define TO_MSG(x)			timeout[TO(x)]
#define PP(x)				(((x) >> 9) & 0x3)
#define PP_MSG(x)			partproc[PP(x)]
#define UU(x)				(((x) >> 8) & 0x3)
#define UU_MSG(x)			uu_msgs[UU(x)]

#define R4(x)				(((x) >> 4) & 0xf)
#define R4_MSG(x)			((R4(x) < 9) ?  memtrans[R4(x)] : "Wrong R4!")

enum tt_ids {
	TT_INSTR = 0,
	TT_DATA,
	TT_GEN,
	TT_RESV,
};

enum ll_ids {
	LL_RESV = 0,
	LL_L1,
	LL_L2,
	LL_LG,
};

enum ii_ids {
	II_MEM = 0,
	II_RESV,
	II_IO,
	II_GEN,
};

enum rrrr_ids {
	R4_GEN	= 0,
	R4_RD,
	R4_WR,
	R4_DRD,
	R4_DWR,
	R4_IRD,
	R4_PREF,
	R4_EVICT,
	R4_SNOOP,
};

#define CASE_AMD_CPUS \
	(cputype == CPU_K8 || \
	 cputype == CPU_F10H || \
 	 cputype == CPU_F11H || \
	 cputype == CPU_F12H || \
 	 cputype == CPU_F14H || \
	 cputype == CPU_F15H || \
 	 cputype == CPU_F16H)
